package controller

import (
	"github.com/gin-gonic/gin"
	"iot_manager_service/app/file/dao"
	"iot_manager_service/app/file/model"
	"iot_manager_service/app/file/service"
	"iot_manager_service/util/cache"
	"iot_manager_service/util/common"
	"iot_manager_service/util/token"
	"net/http"
	"os"
	"path"
	"path/filepath"
	"strconv"
	"strings"
	"time"
)

var FileController = new(file)

type file struct{}

func (f *file) Upload(c *gin.Context) {
	header := c.GetHeader("Authorization")
	claims, _ := token.JwtClaims.ParseJwtToken(header)
	nowSysUser, _ := cache.GetNowSysUser(claims.ID)

	// 获取表单字段的值
	categoryName := c.PostForm("categoryName")
	iv := c.PostForm("iv")
	str := c.PostForm("authId")
	authId := strings.Replace(str, ",", "", -1)

	formFile, err := c.FormFile("file")
	if err != nil {
		c.JSON(http.StatusOK, common.ParamsInvalidResponse("获取文件数据失败", nil))
		return
	}
	//获取后缀
	sufx := path.Ext(formFile.Filename)
	//利用时间戳生成文件名
	fileNameInt := time.Now().Unix()
	fileNameStr := strconv.FormatInt(fileNameInt, 10)
	//新的文件名
	newfileName := fileNameStr + sufx

	currentDir, _ := os.Getwd()
	parentDir := filepath.Dir(currentDir)
	folderPath := filepath.Join(parentDir, "uploadfiles")
	_, err = os.Stat(folderPath)
	if os.IsNotExist(err) {
		os.Mkdir(folderPath, os.ModePerm)
	}
	//保存文件
	filePath := filepath.Join(folderPath, "/", newfileName)
	c.SaveUploadedFile(formFile, filePath)

	savePath := "/uploadfiles/" + newfileName
	currentTimeValue := time.Now()
	upload := service.FileService.Upload(dao.File{
		OriginalName:  strings.TrimSuffix(formFile.Filename, filepath.Ext(formFile.Filename)),
		EncryptedName: formFile.Filename,
		SavePath:      savePath,
		CategoryName:  categoryName,
		AuthId:        authId,
		SuffixName:    path.Ext(formFile.Filename),
		UploadTime:    &currentTimeValue,
		Uploader:      int(nowSysUser.ID),
		Icon:          strings.TrimPrefix(path.Ext(formFile.Filename), ".") + ".png",
		Iv:            iv,
	})
	c.JSON(http.StatusOK, upload)

}

func (f *file) Distribute(c *gin.Context) {
	//获取当前用户
	header := c.GetHeader("Authorization")
	claims, _ := token.JwtClaims.ParseJwtToken(header)
	nowSysUser, _ := cache.GetNowSysUser(claims.ID)

	var reqUserFile model.ReqSysUserFile
	err := c.ShouldBindJSON(&reqUserFile)
	if err != nil {
		c.JSON(http.StatusOK, common.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	distribute := service.FileService.Distribute(&reqUserFile, int(nowSysUser.ID))
	c.JSON(http.StatusOK, distribute)
}

func (f *file) GetSysUserFiles(c *gin.Context) {
	header := c.GetHeader("Authorization")
	claims, _ := token.JwtClaims.ParseJwtToken(header)

	var search model.SearchSysUserFiles
	err := c.ShouldBindJSON(&search)
	if err != nil {
		c.JSON(http.StatusOK, common.ParamsInvalidResponse(err.Error(), nil))
		return
	}

	files := service.FileService.GetSysUserFiles(claims.ID, search)
	c.JSON(http.StatusOK, files)
}

// 转发文件
func (f *file) ForwardingFile(c *gin.Context) {
	fileid := c.Query("fileId")
	header := c.GetHeader("Authorization")
	claims, _ := token.JwtClaims.ParseJwtToken(header)

	requsers := model.ReqSysUsers{}
	if err := c.ShouldBindJSON(&requsers); err != nil {
		c.JSON(http.StatusOK, common.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	uid, _ := strconv.Atoi(claims.ID)
	fid, _ := strconv.Atoi(fileid)
	forwardingFile := service.FileService.ForwardingFile(uid, fid, requsers.SysUsersId)
	c.JSON(http.StatusOK, forwardingFile)
}

func (f *file) GetMyUploadFiles(c *gin.Context) {
	header := c.GetHeader("Authorization")
	claims, _ := token.JwtClaims.ParseJwtToken(header)
	files := service.FileService.GetMyUploadFiles(claims.ID)
	c.JSON(http.StatusOK, files)
}

// 查看文件
func (f *file) ViewFile(c *gin.Context) {
	fileId := c.Query("fileId")
	service.FileService.GetFile(fileId, c)
}

// 下载文件
func (f *file) DownloadFile(c *gin.Context) {
	fileid := c.Query("fileId")
	service.FileService.GetFile(fileid, c)
}
